import pygame
import sys
import math

# 初始化Pygame
pygame.init()

# 设置窗口尺寸
width, height = 800, 600
screen = pygame.display.set_mode((width, height))
pygame.display.set_caption("Bouncing Ball in Triangle")
clock = pygame.time.Clock()

# 定义三角形顶点
triangle_points = [(400, 100), (100, 500), (700, 500)]

# 定义边数据结构（法线向量指向三角形外部）
edges = [
    {  # 左边（A到B）
        "points": [(400, 100), (100, 500)],
        "normal": (0.8, 0.6),  # 单位法线向量
        "length_sq": 250000  # 边长的平方
    },
    {  # 底边（B到C）
        "points": [(100, 500), (700, 500)],
        "normal": (0, -1),  # 单位法线向量
        "length_sq": 360000
    },
    {  # 右边（C到A）
        "points": [(700, 500), (400, 100)],
        "normal": (-0.8, 0.6),  # 单位法线向量
        "length_sq": 250000
    }
]

# 小球属性
ball_radius = 10
ball_pos = [400, 300]
ball_vel = [3, 3]


def calculate_reflection(velocity, normal):
    """计算反射速度"""
    dot_product = velocity[0] * normal[0] + velocity[1] * normal[1]
    return [
        velocity[0] - 2 * dot_product * normal[0],
        velocity[1] - 2 * dot_product * normal[1]
    ]


running = True
while running:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    # 更新小球位置
    ball_pos[0] += ball_vel[0]
    ball_pos[1] += ball_vel[1]

    # 碰撞检测与响应
    for edge in edges:
        start, end = edge["points"]
        nx, ny = edge["normal"]

        # 计算最近点
        ap_x = ball_pos[0] - start[0]
        ap_y = ball_pos[1] - start[1]
        edge_x = end[0] - start[0]
        edge_y = end[1] - start[1]

        t = (ap_x * edge_x + ap_y * edge_y) / edge["length_sq"]
        t = max(0, min(1, t))

        closest_x = start[0] + t * edge_x
        closest_y = start[1] + t * edge_y

        # 计算距离和穿透向量
        dx = ball_pos[0] - closest_x
        dy = ball_pos[1] - closest_y
        distance_sq = dx * dx + dy * dy

        if distance_sq <= ball_radius ** 2:
            penetration = dx * nx + dy * ny
            if penetration < 0:
                # 计算反射速度
                ball_vel[0], ball_vel[1] = calculate_reflection(ball_vel, (nx, ny))

                # 修正位置
                overlap = ball_radius - math.sqrt(distance_sq)
                ball_pos[0] += overlap * nx
                ball_pos[1] += overlap * ny

    # 绘制场景
    screen.fill((255, 255, 255))
    pygame.draw.polygon(screen, (0, 0, 0), triangle_points, 2)
    pygame.draw.circle(screen, (255, 0, 0), (int(ball_pos[0]), int(ball_pos[1])), ball_radius)
    pygame.display.flip()
    clock.tick(60)

pygame.quit()
sys.exit()