import pygame
import random
import math

# 初始化 Pygame
pygame.init()

# 设置窗口尺寸
width, height = 800, 600
screen = pygame.display.set_mode((width, height))
pygame.display.set_caption("三角形内弹跳的小球")

# 定义颜色
white = (255, 255, 255)
black = (0, 0, 0)
red = (255, 0, 0)

# 定义三角形顶点
triangle_points = [(100, 100), (700, 150), (400, 500)]

# 定义小球属性
ball_radius = 10
ball_x = random.randint(150, 650)  # 初始位置随机
ball_y = random.randint(200, 450)  # 初始位置随机
ball_dx = random.uniform(-5, 5)  # 初始速度随机
ball_dy = random.uniform(-5, 5)  # 初始速度随机


# 判断点是否在三角形内部
def point_in_triangle(px, py, triangle_points):
    v0_x, v0_y = triangle_points[0]
    v1_x, v1_y = triangle_points[1]
    v2_x, v2_y = triangle_points[2]

    d0 = (v1_x - v0_x) * (py - v0_y) - (v2_x - v0_x) * (py - v0_y)
    d1 = (v2_x - v0_x) * (py - v0_y) - (v0_x - v0_x) * (py - v0_y)
    d2 = (v0_x - v0_x) * (py - v0_y) - (v1_x - v0_x) * (py - v0_y)

    has_neg = (d0 < 0) or (d1 < 0) or (d2 < 0)
    has_pos = (d0 > 0) or (d1 > 0) or (d2 > 0)

    return not (has_neg and has_pos)


# 碰撞检测函数
def collide_with_triangle(ball_x, ball_y, triangle_points):
    # 使用点到直线距离公式判断是否碰撞
    for i in range(3):
        x1, y1 = triangle_points[i]
        x2, y2 = triangle_points[(i + 1) % 3]

        # 计算点到直线的距离
        distance = abs((y2 - y1) * ball_x - (x2 - x1) * ball_y + x2 * y1 - y2 * x1) / math.sqrt(
            (y2 - y1) ** 2 + (x2 - x1) ** 2)

        if distance <= ball_radius:
            return True, i  # 返回碰撞边索引

    return False, -1


# 游戏循环
running = True
while running:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    # 更新小球位置
    ball_x += ball_dx
    ball_y += ball_dy

    # 判断小球是否超出三角形
    if not point_in_triangle(ball_x, ball_y, triangle_points):
        collided, edge_index = collide_with_triangle(ball_x, ball_y, triangle_points)
        if collided:
            # 计算碰撞点的法线向量
            x1, y1 = triangle_points[edge_index]
            x2, y2 = triangle_points[(edge_index + 1) % 3]
            nx = y2 - y1
            ny = -(x2 - x1)

            # 将速度向量反射
            dot_product = ball_dx * nx + ball_dy * ny
            ball_dx -= 2 * dot_product * nx / (nx ** 2 + ny ** 2)
            ball_dy -= 2 * dot_product * ny / (nx ** 2 + ny ** 2)

    # 绘制背景
    screen.fill(black)

    # 绘制三角形
    pygame.draw.polygon(screen, white, triangle_points, 3)

    # 绘制小球
    pygame.draw.circle(screen, red, (int(ball_x), int(ball_y)), ball_radius)

    # 更新屏幕
    pygame.display.flip()

# 退出 Pygame
pygame.quit()