import pygame
import random
import numpy as np

# 初始化 Pygame
pygame.init()

# 画布大小
WIDTH, HEIGHT = 600, 600
screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("小球在三角形内弹跳")

# 颜色定义
WHITE = (255, 255, 255)
RED = (255, 0, 0)
BLUE = (0, 0, 255)

# 定义三角形顶点
triangle = [(100, 500), (500, 500), (300, 100)]

# 小球属性
ball_radius = 10
ball_x, ball_y = 300, 300  # 初始位置
ball_speed = [random.choice([-3, 3]), random.choice([-3, 3])]  # 初始随机速度

# 计算三角形的边向量和法线
edges = [(triangle[i], triangle[(i + 1) % 3]) for i in range(3)]
normals = []
for (p1, p2) in edges:
    edge_vector = np.array([p2[0] - p1[0], p2[1] - p1[1]])
    normal = np.array([-edge_vector[1], edge_vector[0]])  # 旋转 90 度得到法线
    normal = normal / np.linalg.norm(normal)  # 单位化
    normals.append(normal)

# 判断点是否在三角形内部（重心法）
def is_inside_triangle(x, y):
    def sign(p1, p2, p3):
        return (p1[0] - p3[0]) * (p2[1] - p3[1]) - (p2[0] - p3[0]) * (p1[1] - p3[1])

    d1 = sign((x, y), triangle[0], triangle[1])
    d2 = sign((x, y), triangle[1], triangle[2])
    d3 = sign((x, y), triangle[2], triangle[0])

    has_neg = (d1 < 0) or (d2 < 0) or (d3 < 0)
    has_pos = (d1 > 0) or (d2 > 0) or (d3 > 0)

    return not (has_neg and has_pos)

# 计算小球碰撞后的反射
def reflect(velocity, normal):
    velocity = np.array(velocity)
    normal = np.array(normal)
    return velocity - 2 * np.dot(velocity, normal) * normal

# 主循环
running = True
clock = pygame.time.Clock()

while running:
    screen.fill(WHITE)

    # 画三角形
    pygame.draw.polygon(screen, BLUE, triangle, 2)

    # 更新小球位置
    ball_x += ball_speed[0]
    ball_y += ball_speed[1]

    # 碰撞检测
    for i, ((x1, y1), (x2, y2)) in enumerate(edges):
        edge_vector = np.array([x2 - x1, y2 - y1])
        point_vector = np.array([ball_x - x1, ball_y - y1])
        projection = np.dot(point_vector, edge_vector) / np.dot(edge_vector, edge_vector)
        closest_x, closest_y = x1 + projection * edge_vector[0], y1 + projection * edge_vector[1]

        # 检查最近点是否在线段上
        if 0 <= projection <= 1:
            distance = np.linalg.norm([ball_x - closest_x, ball_y - closest_y])
            if distance <= ball_radius:
                ball_speed = reflect(ball_speed, normals[i])
                break

    # 绘制小球
    pygame.draw.circle(screen, RED, (int(ball_x), int(ball_y)), ball_radius)

    pygame.display.flip()
    clock.tick(60)

    # 事件检测
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

pygame.quit()
