import pygame
import random

# 初始化 Pygame
pygame.init()

# 画布大小
WIDTH, HEIGHT = 600, 400
CELL_SIZE = 20  # 每个格子的大小
GRID_WIDTH = WIDTH // CELL_SIZE
GRID_HEIGHT = HEIGHT // CELL_SIZE

# 颜色定义
WHITE = (255, 255, 255)
GREEN = (0, 255, 0)
RED = (255, 0, 0)
BLUE = (0, 0, 255)
BLACK = (0, 0, 0)

# 方向定义
UP = (0, -1)
DOWN = (0, 1)
LEFT = (-1, 0)
RIGHT = (1, 0)

# 初始化窗口
screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("贪吃蛇")

# 蛇的初始化
snake = [(GRID_WIDTH // 2, GRID_HEIGHT // 2)]
snake_direction = RIGHT  # 初始方向
food = (random.randint(0, GRID_WIDTH - 1), random.randint(0, GRID_HEIGHT - 1))

# 游戏参数
running = True
clock = pygame.time.Clock()
score = 0
speed = 10  # 初始速度

# 生成新食物
def generate_food():
    while True:
        new_food = (random.randint(0, GRID_WIDTH - 1), random.randint(0, GRID_HEIGHT - 1))
        if new_food not in snake:  # 确保食物不会出现在蛇身上
            return new_food

# 显示得分
def draw_score():
    font = pygame.font.Font(None, 30)
    text = font.render(f"Score: {score}", True, WHITE)
    screen.blit(text, (10, 10))

# 主循环
while running:
    screen.fill(BLACK)

    # 事件监听
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False
        elif event.type == pygame.KEYDOWN:
            if event.key == pygame.K_UP and snake_direction != DOWN:
                snake_direction = UP
            elif event.key == pygame.K_DOWN and snake_direction != UP:
                snake_direction = DOWN
            elif event.key == pygame.K_LEFT and snake_direction != RIGHT:
                snake_direction = LEFT
            elif event.key == pygame.K_RIGHT and snake_direction != LEFT:
                snake_direction = RIGHT

    # 计算新蛇头位置
    new_head = (snake[0][0] + snake_direction[0], snake[0][1] + snake_direction[1])

    # 撞墙检测
    if new_head[0] < 0 or new_head[0] >= GRID_WIDTH or new_head[1] < 0 or new_head[1] >= GRID_HEIGHT:
        running = False

    # 撞到自己
    if new_head in snake:
        running = False

    # 添加新的头部
    snake.insert(0, new_head)

    # 吃食物
    if new_head == food:
        score += 10
        speed += 0.5  # 加速
        food = generate_food()
    else:
        snake.pop()  # 移除尾部保持长度

    # 画蛇
    for segment in snake:
        pygame.draw.rect(screen, GREEN, (segment[0] * CELL_SIZE, segment[1] * CELL_SIZE, CELL_SIZE, CELL_SIZE))

    # 画食物
    pygame.draw.rect(screen, RED, (food[0] * CELL_SIZE, food[1] * CELL_SIZE, CELL_SIZE, CELL_SIZE))

    # 显示得分
    draw_score()

    pygame.display.flip()
    clock.tick(speed)

pygame.quit()
