-- 创建数据库 (如果数据库不存在)
CREATE DATABASE IF NOT EXISTS library_management;

-- 使用数据库
USE library_management;

-- 创建分类表 (Category)
CREATE TABLE Category (
  category_id INT AUTO_INCREMENT PRIMARY KEY,
  category_name VARCHAR(255) NOT NULL
);

-- 创建图书表 (Book)
CREATE TABLE Book (
  book_id INT AUTO_INCREMENT PRIMARY KEY,
  category_id INT,
  title VARCHAR(255) NOT NULL,
  author VARCHAR(255) NOT NULL,
  publisher VARCHAR(255),
  isbn VARCHAR(20) UNIQUE,
  publication_date DATE,
  price DECIMAL(10, 2),
  stock INT,
  FOREIGN KEY (category_id) REFERENCES Category(category_id)
);

-- 创建读者表 (Reader)
CREATE TABLE Reader (
  reader_id INT AUTO_INCREMENT PRIMARY KEY,
  name VARCHAR(255) NOT NULL,
  gender VARCHAR(10),
  phone VARCHAR(20) UNIQUE,
  address VARCHAR(255)
);

-- 创建借阅记录表 (Borrowing)
CREATE TABLE Borrowing (
  borrowing_id INT AUTO_INCREMENT PRIMARY KEY,
  book_id INT,
  reader_id INT,
  borrow_date DATE,
  return_date DATE,
  FOREIGN KEY (book_id) REFERENCES Book(book_id),
  FOREIGN KEY (reader_id) REFERENCES Reader(reader_id)
);

-- 创建管理员表 (Admin)
CREATE TABLE Admin (
  admin_id INT AUTO_INCREMENT PRIMARY KEY,
  username VARCHAR(255) UNIQUE,
  password VARCHAR(255) NOT NULL
);

-- 插入示例数据 (可选)
INSERT INTO Category (category_name) VALUES
('文学'), ('计算机'), ('历史'), ('哲学');

INSERT INTO Book (category_id, title, author, isbn, publication_date, price, stock) VALUES
(1, '《红楼梦》', '曹雪芹', '978-7-02-014010-1', '1754-01-01', 98.00, 10),
(2, '《Java编程思想》', 'Bruce Eckel', '978-0-13-187256-2', '2006-06-05', 128.00, 5),
(3, '《中国近代史》', '李剑农', '978-7-301-21083-2', '2012-01-01', 68.00, 8);

INSERT INTO Reader (name, gender, phone, address) VALUES
('张三', '男', '13812345678', '北京市海淀区'),
('李四', '女', '13987654321', '上海市徐汇区');

INSERT INTO Borrowing (book_id, reader_id, borrow_date) VALUES
(1, 1, '2024-07-20'),
(2, 2, '2024-07-21');

INSERT INTO Admin (username, password) VALUES
('admin', '123456');